@@ -60,6 +60,7 @@ def main():
         num_refinement_steps=5,
         temperature=0.8,
         top_k=50,
+        repetition_penalty=1.2,  # Prevent token repetition
         use_adaptive_steps=True,
         use_torch_compile=True if device == "cuda" else False # Enable torch.compile for speed
     )
@@ -81,12 +82,13 @@ def main():
     # Warmup
     print("Warming up...")
     with torch.no_grad():
-        _ = generator.generate(input_ids, max_new_tokens=10)
+        _ = generator.generate(input_ids, max_new_tokens=10, use_autoregressive=True)
 
     # Actual generation
     print("Generating...")
+    print("Mode: Autoregressive (generates tokens one-by-one for coherent output)")
     with torch.no_grad():
-        output_ids = generator.generate(input_ids)
+        output_ids = generator.generate(input_ids, use_autoregressive=True)
 
     output_text = tokenizer.decode(output_ids[0], skip_special_tokens=True)
     
