@@ -7,6 +7,20 @@ import sys
 import warnings
 from typing import Optional, Any
 
+# Suppress optional dependency warnings - they clutter output
+# Users will see errors if they try to use features that need these
+warnings.filterwarnings('ignore', category=UserWarning, module='parallel_llm.compat')
+
+# Suppress Pydantic warnings from dependencies (transformers, etc.)
+try:
+    # Try to import PydanticUserWarning if available
+    from pydantic import PydanticUserWarning
+    warnings.filterwarnings('ignore', category=PydanticUserWarning)
+except ImportError:
+    pass
+# Generic filter for the specific message if class import fails or for older versions
+warnings.filterwarnings('ignore', message=".*UnsupportedFieldAttributeWarning.*")
+
 # Platform detection
 PLATFORM = sys.platform
 IS_WINDOWS = PLATFORM.startswith('win')
@@ -21,50 +35,44 @@ is_macos = IS_MACOS
 def try_import(module_name: str, package_name: Optional[str] = None) -> Any:
     """
     Try to import a module, returning None if import fails.
-
+    
     Args:
         module_name: The module to import
         package_name: Optional package name for better error messages
-
+    
     Returns:
         The imported module or None if import fails
     """
     try:
         __import__(module_name)
         return sys.modules[module_name]
-    except ImportError as e:
-        if package_name:
-            warnings.warn(
-                f"Optional dependency '{package_name}' not available: {e}. "
-                f"Some features may be limited. Install with: pip install {package_name}"
-            )
-        else:
-            warnings.warn(f"Could not import {module_name}: {e}")
+    except ImportError:
+        # Silently return None - no warnings
         return None
 
 # Core dependencies (lazy loading to avoid import errors)
 torch = None  # Will be loaded on demand
 numpy = None  # Will be loaded on demand
 
-# Optional GPU dependencies
-triton = try_import('triton', 'triton')
-flash_attn = try_import('flash_attn', 'flash-attn')
-xformers = try_import('xformers', 'xformers')
+# Optional GPU dependencies (no warnings if not available)
+triton = try_import('triton')
+flash_attn = try_import('flash_attn')
+xformers = try_import('xformers')
 
-# Optional distributed training
-deepspeed = try_import('deepspeed', 'deepspeed')
+# Optional distributed training (no warnings)
+deepspeed = try_import('deepspeed')
 
-# Optional inference optimization
-vllm = try_import('vllm', 'vllm')
+# Optional inference optimization (no warnings)
+vllm = try_import('vllm')
 
-# Optional multimodal
-PIL = try_import('PIL', 'Pillow')
-requests = try_import('requests', 'requests')
-timm = try_import('timm', 'timm')
+# Optional multimodal (no warnings)
+PIL = try_import('PIL')
+requests = try_import('requests')
+timm = try_import('timm')
 
-# Optional logging
-wandb = try_import('wandb', 'wandb')
-tensorboard = try_import('tensorboard', 'tensorboard')
+# Optional logging (no warnings)
+wandb = try_import('wandb')
+tensorboard = try_import('tensorboard')
 
 def check_pytorch_cuda():
     """Check if PyTorch CUDA is available."""
