@@ -208,7 +208,7 @@ class ParallelGenerator:
             self.static_output, _ = self.model(self.static_input, self.static_timestep)
 
     @torch.no_grad()
-    def generate(
+    def autoregressive_generate(
         self,
         prompt_tokens: torch.Tensor,
         max_new_tokens: Optional[int] = None,
@@ -216,7 +216,8 @@ class ParallelGenerator:
         **kwargs
     ) -> torch.Tensor:
         """
-        Generate tokens in parallel
+        Generate tokens autoregressively (one at a time, left to right)
+        This produces coherent sequences where each token depends on previous ones.
 
         Args:
             prompt_tokens: [batch, prompt_len] input tokens
@@ -231,6 +232,140 @@ class ParallelGenerator:
         max_new_tokens = max_new_tokens or self.config.max_new_tokens
         device = prompt_tokens.device
 
+        # Validate prompt tokens
+        max_token_id = prompt_tokens.max().item()
+        if max_token_id >= self.model.config.vocab_size:
+            print(
+                f"Warning: Prompt contains token IDs ({max_token_id}) greater than model vocabulary size "
+                f"({self.model.config.vocab_size}). Resizing model embeddings..."
+            )
+            new_vocab_size = ((max_token_id + 1 + 63) // 64) * 64
+            self.model.resize_token_embeddings(new_vocab_size)
+            print(f"Model embeddings resized to {new_vocab_size}")
+            self.model.config.vocab_size = new_vocab_size
+
+        # Start with prompt
+        generated = prompt_tokens.clone()
+
+        # Generate tokens one at a time
+        for i in range(max_new_tokens):
+            # Create input with one MASK token at the end
+            mask_token_id = getattr(self.model, 'mask_token_id', 0)
+            current_input = torch.cat([
+                generated,
+                torch.full((batch_size, 1), mask_token_id, dtype=torch.long, device=device)
+            ], dim=1)
+
+            # Run multiple refinement steps for this single position
+            for step in range(self.config.num_refinement_steps):
+                timestep = torch.full(
+                    (batch_size,), 
+                    self.config.num_refinement_steps - step, 
+                    device=device, 
+                    dtype=torch.long
+                )
+
+                # Forward pass
+                model_kwargs = {'return_confidence': True}
+                if pixel_values is not None:
+                    model_kwargs['pixel_values'] = pixel_values
+                
+                output = self.model(current_input, timestep, **model_kwargs)
+                if isinstance(output, tuple):
+                    logits, confidence = output
+                else:
+                    logits = output
+                    confidence = None
+
+                # Get logits for the last (masked) position
+                next_token_logits = logits[:, -1, :]
+
+                # Apply temperature
+                if self.config.temperature > 0:
+                    next_token_logits = next_token_logits / self.config.temperature
+                
+                # Replace NaN and inf
+                next_token_logits = torch.nan_to_num(
+                    next_token_logits, nan=-1e9, posinf=1e9, neginf=-1e9
+                )
+
+                # Apply top-k filtering
+                if self.config.top_k > 0:
+                    next_token_logits = self._top_k_filtering(
+                        next_token_logits.unsqueeze(1), self.config.top_k
+                    ).squeeze(1)
+
+                # Apply top-p filtering
+                if self.config.top_p < 1.0:
+                    next_token_logits = self._top_p_filtering(
+                        next_token_logits.unsqueeze(1), self.config.top_p
+                    ).squeeze(1)
+
+                # Apply repetition penalty
+                if self.config.repetition_penalty != 1.0:
+                    for b in range(batch_size):
+                        for prev_token in set(generated[b].tolist()):
+                            if 0 <= prev_token < self.model.config.vocab_size:
+                                if next_token_logits[b, prev_token] < 0:
+                                    next_token_logits[b, prev_token] *= self.config.repetition_penalty
+                                else:
+                                    next_token_logits[b, prev_token] /= self.config.repetition_penalty
+
+                # Sample next token
+                probs = F.softmax(next_token_logits, dim=-1)
+                probs = torch.nan_to_num(probs, nan=0.0, posinf=1.0, neginf=0.0)
+                probs_sum = probs.sum(dim=-1, keepdim=True)
+                probs = probs / (probs_sum + 1e-10)
+                
+                next_token = torch.multinomial(probs, num_samples=1)
+                next_token = torch.clamp(next_token, 0, self.model.config.vocab_size - 1)
+
+                # Update the last position
+                current_input[:, -1] = next_token.squeeze(-1)
+
+                # Check confidence for early stopping
+                if confidence is not None:
+                    pos_confidence = confidence[:, -1]
+                    if (pos_confidence > self.config.confidence_threshold).all():
+                        break
+
+            # Add the new token to generated sequence
+            generated = torch.cat([generated, next_token], dim=1)
+
+        return generated
+
+    @torch.no_grad()
+    def generate(
+        self,
+        prompt_tokens: torch.Tensor,
+        max_new_tokens: Optional[int] = None,
+        pixel_values: Optional[torch.Tensor] = None,
+        use_autoregressive: bool = True,  # NEW: Toggle between modes
+        **kwargs
+    ) -> torch.Tensor:
+        """
+        Generate tokens either autoregressively or in parallel
+
+        Args:
+            prompt_tokens: [batch, prompt_len] input tokens
+            max_new_tokens: maximum number of tokens to generate
+            pixel_values: [batch, channels, height, width] for multimodal generation
+            use_autoregressive: if True, generate tokens one-by-one for coherent output
+            **kwargs: additional arguments passed to the model
+
+        Returns:
+            generated_tokens: [batch, prompt_len + max_new_tokens]
+        """
+        if use_autoregressive:
+            return self.autoregressive_generate(
+                prompt_tokens, max_new_tokens, pixel_values, **kwargs
+            )
+        
+        # Original parallel generation code
+        batch_size, prompt_len = prompt_tokens.shape
+        max_new_tokens = max_new_tokens or self.config.max_new_tokens
+        device = prompt_tokens.device
+
         # Validate prompt tokens against vocab size and resize if necessary
         max_token_id = prompt_tokens.max().item()
         if max_token_id >= self.model.config.vocab_size:
