@@ -39,37 +39,62 @@ class DistributedTrainer:
         train_dataloader,
         eval_dataloader=None,
     ):
+        logger.info("Initializing DistributedTrainer...")
         self.model = model
         self.train_config = train_config
         self.model_config = model_config
         self.train_dataloader = train_dataloader
         self.eval_dataloader = eval_dataloader
+        self.use_data_parallel = False  # Flag for DataParallel mode
 
         # Initialize distributed
+        logger.info("Step 1/7: Initializing distributed environment...")
         self._init_distributed()
+        logger.info("✓ Distributed environment initialized")
 
         # Setup model for distributed training
+        logger.info("Step 2/7: Setting up distributed model...")
         self._setup_distributed_model()
+        logger.info("✓ Distributed model setup complete")
 
         # Setup optimizer and scheduler
+        logger.info("Step 3/7: Setting up optimizer...")
         self._setup_optimizer()
+        logger.info("✓ Optimizer created")
+
+        logger.info("Step 4/7: Setting up learning rate scheduler...")
         self._setup_scheduler()
+        logger.info("✓ Scheduler created")
 
         # Setup mixed precision
+        logger.info("Step 5/7: Setting up mixed precision...")
         self._setup_mixed_precision()
+        logger.info("✓ Mixed precision configured")
 
         # Setup loss function
+        logger.info("Step 6/7: Setting up loss function...")
         self._setup_loss()
+        logger.info("✓ Loss function ready")
 
         # Compile model if requested (disabled by default for CUDA compatibility)
         if getattr(train_config, 'use_torch_compile', False):
+            logger.info("Compiling model with torch.compile...")
             self._compile_model()
+            logger.info("✓ Model compilation complete")
 
         # Setup logging
+        logger.info("Step 7/7: Setting up training logs...")
         self._setup_logging()
+        logger.info("✓ Logging configured")
 
         self.global_step = 0
         self.epoch = 0
+        logger.info("="*60)
+        logger.info("✅ DistributedTrainer initialization complete!")
+        logger.info(f"Training on: {self.device}")
+        logger.info(f"World size: {self.world_size}")
+        logger.info(f"Using DataParallel: {self.use_data_parallel}")
+        logger.info("="*60)
 
     def _init_distributed(self):
         """Initialize distributed training or single-device training"""
@@ -107,29 +132,15 @@ class DistributedTrainer:
                     self.is_distributed = False
                     
         elif has_multi_gpu:
-            # Auto-initialize for Kaggle/Colab with multiple GPUs
-            logger.info(f"🚀 Detected {num_gpus} GPUs. Auto-initializing distributed training...")
-            logger.info("💡 Tip: For more control, use: torchrun --nproc_per_node={num_gpus} script.py")
+            # Use DataParallel for Kaggle/Colab multi-GPU (single process)
+            logger.info(f"🎮 Detected {num_gpus} GPUs. Using DataParallel for multi-GPU training...")
+            logger.info("💡 For true distributed training, use: torchrun --nproc_per_node=N script.py")
             
-            # Set up environment variables for torch.distributed
-            os.environ.setdefault("MASTER_ADDR", "localhost")
-            os.environ.setdefault("MASTER_PORT", "12355")
-            os.environ.setdefault("RANK", "0")
-            os.environ.setdefault("WORLD_SIZE", str(num_gpus))
-            os.environ.setdefault("LOCAL_RANK", "0")
+            # Don't try to fake distributed training - use DataParallel instead
+            self.is_distributed = False
+            self.use_data_parallel = True
             
-            self.is_distributed = True
-            try:
-                # Add timeout to fail fast
-                dist.init_process_group(
-                    backend="nccl", 
-                    init_method="env://",
-                    timeout=datetime.timedelta(seconds=30)
-                )
-            except Exception as e:
-                logger.warning(f"Failed to auto-initialize distributed training: {e}")
-                logger.info("Falling back to single-device training")
-                self.is_distributed = False
+            # No need to call dist.init_process_group - DataParallel handles it
         else:
             # Single device
             self.is_distributed = False
@@ -159,6 +170,13 @@ class DistributedTrainer:
         logger.info("Setting up model on devices...")
         self.model = self.model.to(self.device)
 
+        # Check if we should use DataParallel
+        if self.use_data_parallel and torch.cuda.device_count() > 1:
+            logger.info(f"Wrapping model with DataParallel for {torch.cuda.device_count()} GPUs...")
+            self.model = torch.nn.DataParallel(self.model)
+            logger.info("✓ DataParallel initialized - model will use all available GPUs")
+            return
+
         if not self.is_distributed:
             # Single device - no distributed wrapper needed
             logger.info("✓ Model ready on single device")
@@ -301,67 +319,86 @@ class DistributedTrainer:
     def _setup_optimizer(self):
         """Setup AdamW optimizer"""
         if self.train_config.use_deepspeed:
+            logger.info("Skipping optimizer setup (DeepSpeed will handle it)")
             return  # DeepSpeed handles optimizer
 
-        # Separate parameters with and without weight decay
-        decay_params = []
-        no_decay_params = []
+        try:
+            logger.info("Collecting model parameters for optimizer...")
+            # Separate parameters with and without weight decay
+            decay_params = []
+            no_decay_params = []
 
-        for name, param in self.model.named_parameters():
-            if param.requires_grad:
-                if "bias" in name or "norm" in name or "embed" in name:
-                    no_decay_params.append(param)
-                else:
-                    decay_params.append(param)
+            param_count = 0
+            for name, param in self.model.named_parameters():
+                if param.requires_grad:
+                    param_count += 1
+                    if "bias" in name or "norm" in name or "embed" in name:
+                        no_decay_params.append(param)
+                    else:
+                        decay_params.append(param)
 
-        optimizer_grouped_parameters = [
-            {"params": decay_params, "weight_decay": self.train_config.weight_decay},
-            {"params": no_decay_params, "weight_decay": 0.0},
-        ]
+            logger.info(f"Found {param_count} trainable parameters ({len(decay_params)} with decay, {len(no_decay_params)} without)")
 
-        self.optimizer = torch.optim.AdamW(
-            optimizer_grouped_parameters,
-            lr=self.train_config.learning_rate,
-            betas=(self.train_config.adam_beta1, self.train_config.adam_beta2),
-            eps=self.train_config.adam_epsilon,
-        )
+            optimizer_grouped_parameters = [
+                {"params": decay_params, "weight_decay": self.train_config.weight_decay},
+                {"params": no_decay_params, "weight_decay": 0.0},
+            ]
+
+            logger.info("Creating AdamW optimizer...")
+            self.optimizer = torch.optim.AdamW(
+                optimizer_grouped_parameters,
+                lr=self.train_config.learning_rate,
+                betas=(self.train_config.adam_beta1, self.train_config.adam_beta2),
+                eps=self.train_config.adam_epsilon,
+            )
+            logger.info(f"Optimizer created with learning rate: {self.train_config.learning_rate}")
+        except Exception as e:
+            logger.error(f"Failed to setup optimizer: {e}")
+            raise
 
     def _setup_scheduler(self):
         """Setup learning rate scheduler"""
         if self.train_config.use_deepspeed:
+            logger.info("Skipping scheduler setup (DeepSpeed will handle it)")
             return  # DeepSpeed handles scheduler
 
-        # Calculate main scheduler iterations (ensure it's positive)
-        main_iters = max(1, self.train_config.num_train_steps - self.train_config.warmup_steps)
+        try:
+            # Calculate main scheduler iterations (ensure it's positive)
+            main_iters = max(1, self.train_config.num_train_steps - self.train_config.warmup_steps)
+            logger.info(f"Setting up {self.train_config.lr_scheduler} scheduler for {main_iters} iterations...")
 
-        if self.train_config.lr_scheduler == "cosine":
-            self.scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(
-                self.optimizer,
-                T_max=main_iters,
-            )
-        elif self.train_config.lr_scheduler == "linear":
-            self.scheduler = torch.optim.lr_scheduler.LinearLR(
-                self.optimizer,
-                start_factor=1.0,
-                end_factor=0.1,  # Don't go to 0.0 to avoid issues
-                total_iters=main_iters,
-            )
-        else:
-            self.scheduler = torch.optim.lr_scheduler.ConstantLR(self.optimizer, factor=1.0)
+            if self.train_config.lr_scheduler == "cosine":
+                self.scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(
+                    self.optimizer,
+                    T_max=main_iters,
+                )
+            elif self.train_config.lr_scheduler == "linear":
+                self.scheduler = torch.optim.lr_scheduler.LinearLR(
+                    self.optimizer,
+                    start_factor=1.0,
+                    end_factor=0.1,  # Don't go to 0.0 to avoid issues
+                    total_iters=main_iters,
+                )
+            else:
+                self.scheduler = torch.optim.lr_scheduler.ConstantLR(self.optimizer, factor=1.0)
 
-        # Add warmup
-        if self.train_config.warmup_steps > 0:
-            warmup_scheduler = torch.optim.lr_scheduler.LinearLR(
-                self.optimizer,
-                start_factor=0.1,  # Start from small value, not 0.0
-                end_factor=1.0,
-                total_iters=self.train_config.warmup_steps,
-            )
-            self.scheduler = torch.optim.lr_scheduler.SequentialLR(
-                self.optimizer,
-                schedulers=[warmup_scheduler, self.scheduler],
-                milestones=[self.train_config.warmup_steps],
-            )
+            # Add warmup
+            if self.train_config.warmup_steps > 0:
+                logger.info(f"Adding warmup for {self.train_config.warmup_steps} steps...")
+                warmup_scheduler = torch.optim.lr_scheduler.LinearLR(
+                    self.optimizer,
+                    start_factor=0.1,  # Start from small value, not 0.0
+                    end_factor=1.0,
+                    total_iters=self.train_config.warmup_steps,
+                )
+                self.scheduler = torch.optim.lr_scheduler.SequentialLR(
+                    self.optimizer,
+                    schedulers=[warmup_scheduler, self.scheduler],
+                    milestones=[self.train_config.warmup_steps],
+                )
+        except Exception as e:
+            logger.error(f"Failed to setup scheduler: {e}")
+            raise
 
     def _setup_loss(self):
         """Setup loss function for diffusion training"""
@@ -395,24 +432,69 @@ class DistributedTrainer:
 
     def _setup_logging(self):
         """Setup logging and monitoring"""
-        if self.rank == 0 and self.train_config.use_wandb:
-            wandb.init(
-                project=self.train_config.wandb_project,
-                config={
-                    "model": self.model_config.__dict__,
-                    "training": self.train_config.__dict__,
-                },
-            )
+        if self.rank == 0 and getattr(self.train_config, 'use_wandb', False):
+            try:
+                logger.info("Initializing Weights & Biases (may take few seconds)...")
+                
+                # Try to import signal for timeout (works on Linux/Kaggle, not Windows)
+                try:
+                    import signal
+                    
+                    # Set 10-second timeout for wandb init
+                    def timeout_handler(signum, frame):
+                        raise TimeoutError("Wandb initialization timed out after 10 seconds")
+                    
+                    signal.signal(signal.SIGALRM, timeout_handler)
+                    signal.alarm(10)  # 10 second timeout
+                    
+                    try:
+                        wandb.init(
+                            project=self.train_config.wandb_project,
+                            config={
+                                "model": self.model_config.__dict__,
+                                "training": self.train_config.__dict__,
+                            },
+                        )
+                        logger.info("✓ Wandb initialized successfully")
+                    finally:
+                        signal.alarm(0)  # Cancel timeout
+                        
+                except (ImportError, AttributeError, TimeoutError) as e:
+                    # signal module not available (Windows) or timeout occurred
+                    logger.warning(f"Wandb initialization failed or timed out: {e}")
+                    logger.info("Continuing training without wandb logging...")
+                    
+            except Exception as e:
+                logger.warning(f"Wandb setup failed: {e}. Continuing without wandb...")
+        else:
+            if not getattr(self.train_config, 'use_wandb', False):
+                logger.info("Wandb logging disabled in config")
+            else:
+                logger.info("Wandb logging skipped (not rank 0)")
 
     def train(self):
         """Main training loop"""
-        self.model.train()
+        logger.info("Starting training...")
+        
+        # Create progress bar with clean single-line updates
+        pbar = tqdm(
+            total=self.train_config.num_train_steps,
+            desc="Training",
+            disable=self.rank != 0,
+            dynamic_ncols=True,
+            position=0,
+            leave=True,
+            mininterval=0.5,  # Update at most twice per second
+            bar_format='{l_bar}{bar}| {n_fmt}/{total_fmt} [{elapsed}<{remaining}, {rate_fmt}]'
+        )
 
-        if self.rank == 0:
-            progress_bar = tqdm(total=self.train_config.num_train_steps, desc="Training")
+        self.model.train()
 
         while self.global_step < self.train_config.num_train_steps:
             for batch in self.train_dataloader:
+                if self.global_step >= self.train_config.num_train_steps:
+                    break
+
                 # Move batch to device
                 batch = {k: v.to(self.device) if isinstance(v, torch.Tensor) else v 
                         for k, v in batch.items()}
@@ -433,7 +515,6 @@ class DistributedTrainer:
                             self.train_config.max_grad_norm
                         )
                         self.scaler.step(self.optimizer)
-                        self.scaler.update()
                     else:
                         loss.backward()
                         torch.nn.utils.clip_grad_norm_(
@@ -460,14 +541,18 @@ class DistributedTrainer:
                     self.save_checkpoint()
 
                 if self.rank == 0:
-                    progress_bar.update(1)
+                    pbar.update(1)
+                    pbar.set_postfix({
+                        'loss': f'{loss.item():.4f}',
+                        'lr': f'{self.scheduler.get_last_lr()[0]:.2e}'
+                    })
 
                 if self.global_step >= self.train_config.num_train_steps:
                     break
 
         if self.rank == 0:
-            progress_bar.close()
-
+            pbar.close()
+            logger.info("Training completed!")
     def training_step(self, batch: Dict[str, torch.Tensor]) -> torch.Tensor:
         """Single training step for diffusion training"""
         input_ids = batch['input_ids']  # (batch, seq_len)
@@ -544,32 +629,43 @@ class DistributedTrainer:
 
     def save_checkpoint(self):
         """Save model checkpoint"""
-        if self.rank == 0:
-            checkpoint_dir = Path(self.train_config.output_dir) / f"checkpoint-{self.global_step}"
-            checkpoint_dir.mkdir(parents=True, exist_ok=True)
+        if self.rank != 0:
+            return  # Only rank 0 saves
 
-            # Save model state
-            if self.train_config.use_fsdp:
-                from torch.distributed.fsdp import FullStateDictConfig, StateDictType
+        checkpoint_dir = Path(self.train_config.output_dir) / f"checkpoint-{self.global_step}"
+        checkpoint_dir.mkdir(parents=True, exist_ok=True)
 
-                save_policy = FullStateDictConfig(offload_to_cpu=True, rank0_only=True)
-                with FSDP.state_dict_type(self.model, StateDictType.FULL_STATE_DICT, save_policy):
-                    state_dict = self.model.state_dict()
-                    torch.save(state_dict, checkpoint_dir / "pytorch_model.bin")
-            else:
+        logger.info(f"Saving checkpoint to {checkpoint_dir}")
+
+        # Save model state - handle different model types
+        if self.use_data_parallel:
+            # DataParallel: unwrap model.module
+            model_to_save = self.model.module if hasattr(self.model, 'module') else self.model
+            torch.save(model_to_save.state_dict(), checkpoint_dir / "pytorch_model.bin")
+            
+        elif self.train_config.use_fsdp and self.is_distributed:
+            # FSDP: use special handling
+            from torch.distributed.fsdp import FullStateDictConfig, StateDictType, FullyShardedDataParallel as FSDP
+            save_policy = FullStateDictConfig(offload_to_cpu=True, rank0_only=True)
+            with FSDP.state_dict_type(self.model, StateDictType.FULL_STATE_DICT, save_policy):
                 torch.save(self.model.state_dict(), checkpoint_dir / "pytorch_model.bin")
+                
+        else:
+            # Regular model or DDP: unwrap if needed
+            model_to_save = self.model.module if hasattr(self.model, 'module') else self.model
+            torch.save(model_to_save.state_dict(), checkpoint_dir / "pytorch_model.bin")
 
-            # Save optimizer and scheduler
-            torch.save(self.optimizer.state_dict(), checkpoint_dir / "optimizer.pt")
-            torch.save(self.scheduler.state_dict(), checkpoint_dir / "scheduler.pt")
+        # Save optimizer and scheduler
+        torch.save(self.optimizer.state_dict(), checkpoint_dir / "optimizer.pt")
+        torch.save(self.scheduler.state_dict(), checkpoint_dir / "scheduler.pt")
 
-            # Save training state
-            torch.save({
-                "global_step": self.global_step,
-                "epoch": self.epoch,
-            }, checkpoint_dir / "training_state.pt")
+        # Save training state
+        torch.save({
+            "global_step": self.global_step,
+            "epoch": self.epoch,
+        }, checkpoint_dir / "training_state.pt")
 
-            logger.info(f"Saved checkpoint to {checkpoint_dir}")
+        logger.info(f"✓ Checkpoint saved at step {self.global_step}")
 
     def _log_metrics(self, metrics: Dict[str, Any]):
         """Log metrics to wandb"""
